/* $Id: binfmt.h,v 1.4 2000/11/10 03:06:37 malekith Exp $ */

/* 
 * definition of binary file format used both for .ok objects
 * and executables for vm 
 */

/* if file starts with `#!', characters till '\n' shall be ignored
 * by vm. 
 * next there should be 4 byte magic :
 */

#define KBIN_FILE_MAGIC		"KBIN"
#define KBIN_TAG_MAGIC		"TAG"

/* note that if head->version > KBIN_VERSION_CURRENT, other header fields
 * can be diffrent -> stop parsing ! */
#define KBIN_VERSION_1		1
#define KBIN_VERSION_CURRENT	KBIN_VERSION_1	

#define KBIN_BIG_ENDIAN		1
#define KBIN_LITTLE_ENDIAN	2

#ifdef WORDS_BIGENDIAN
#	define KBIN_HOST_ENDIAN KBIN_BIG_ENDIAN
#else
#	define KBIN_HOST_ENDIAN KBIN_LITTLE_ENDIAN
#endif

/* after that file header : */
struct kbin_file_head {
	byte magic[4];		/* "KBIN" */
	byte version;		/* always KBIN_VERSION_1 at the moment */
	byte endian;		/* KBIN_{LITTLE,BIG}_ENDIAN */
	byte iptr_size;		/* sizeof(iptr) */
	byte real_size;		/* sizeof(real) -- maybe some real type ? */
};

/* and then tags with following heading */
struct kbin_tag_head {
	byte magic[3];		/* "TAG" */
	byte tag;		/* one of kbin_tag_XXX */
	u32 length;		/* len of data, w/o magic and length fields */
	/* data[] follows */
};

enum {
	kbin_tag_error,		/* should not occur */
	kbin_tag_ext,		/* extension for future use - generate error */
	kbin_tag_eof,		/* marks end of file - stop processing */
	kbin_tag_comment,	/* don't parse ... */
	
	/* list of used classes: 
	 *	u32 no
	 *	ClassNo[0]\0ClassNo[1]\0....ClassNo[no-1]\0
	 */
	kbin_tag_ok_classes,
	/* list of offsets at which classes were used
	 * at this offsets there should be u32 < ok_classes->no
	 * to be mapped */
	kbin_tag_ok_class_xrefs,
	/* list of defined functions:
	 * struct { u32 class_id, func_id, offset } [] */
	kbin_tag_ok_funcs,
	/* list of classes functions - in which class definition
	 * could be found :
	 * u32 nclasses;
	 * { u32 nfuncs; { u32 class_id } [ nfuncs ] } [ nclasses ] */
	kbin_tag_ok_funcs_xrefs,

	kbin_tag_code,		/* start code segment, there can be only 
				 * one in entire file 
				 */
	kbin_tag_vtables,	/* u32 nclasses; 
				 * { u32 nfuncs; u32 funcs_offsets[funcs];
				 * } [ nclasses ];
				 */
	
	kbin_tag_last		/* only marker ... */
};

/* utility functions: */

struct kbin_tag {
	int id;
	int len;
	byte *data;
};

struct kbin_file {
	int fd;
	int flags;
	int error;
	char *name;
	
	byte *buf;
	int len, off;
};

/* 
 * all functions returing ptrs, returns 0 in case of error.
 * returning ints serves no other purpouse then returning back
 * status. non-zero means error.
 * most of functions will print some error msg on stderr
 */

/* 
 * open given file, if (ro) {check it header} else {init it with fline,
 * which should be sth like "#! /usr/local/bin/kelpie-vm\n", and
 * proper header}
 */
struct kbin_file *kbin_open(const char *fn, int ro, const char *fline);

/* 
 * close file and free any associated memory 
 */
int kbin_close(struct kbin_file *f);

/*
 * get next tag from file, return it in allocated space
 * you can safly copy tag->data ptr somewhere, but don't forget
 * to set it to NULL in tag and to free() it later
 */
struct kbin_tag *kbin_get_tag(struct kbin_file *f);

/*
 * free memory allocated by kbin_get_tag().
 */
int kbin_kill_tag(struct kbin_tag *t);

/*
 * add given tag to file
 */
int kbin_add_tag(struct kbin_file *f, int id, int len, const byte *data);

/*
 * add kbin_tag_eof to file and close it.
 */
int kbin_fini(struct kbin_file *f);
